import { MainAreaWidget } from '@jupyterlab/apputils';
import { ContentHeaderWidget } from './widgets/ContentHeaderToolbar';
import { requestAPI } from './handler';
export class TransformPollingManager {
    constructor(props) {
        this._docManager = props.docManager;
        this._app = props.app;
        this.polling = {
            id: '',
            filename: '',
            dirname: '',
            status: false,
            count: 0
        };
        this.pollingCache = {
            id: '',
            filename: '',
            dirname: '',
            status: false,
            count: 0
        };
        this.startPolling = this.startPolling.bind(this);
        this.openTransformedFile = this.openTransformedFile.bind(this);
        this.acceptFile = this.acceptFile.bind(this);
        this.declineFile = this.declineFile.bind(this);
    }
    startPolling(uid, filename, dirname) {
        if (this.polling.status) {
            return;
        }
        this.polling = {
            id: uid,
            filename: filename,
            dirname: dirname,
            status: true,
            count: 0
        };
        const dataToSend = { uid, dirname };
        const pollingRef = setInterval(() => {
            const reply = requestAPI('CHECK_STATUS', {
                body: JSON.stringify(dataToSend),
                method: 'POST'
            });
            this.polling = Object.assign(Object.assign({}, this.polling), { count: this.polling.count + 1 });
            reply
                .then((response) => {
                if (response.status === 'finished') {
                    this.pollingCache = Object.assign({}, this.polling);
                    this.polling = {
                        id: '',
                        filename: '',
                        dirname: '',
                        status: false,
                        count: 0
                    };
                    clearInterval(pollingRef);
                    this.openTransformedFile(response.file);
                }
                else {
                    if (this.polling.count >= 60) {
                        this.pollingCache = {
                            id: '',
                            filename: '',
                            dirname: '',
                            status: false,
                            count: 0
                        };
                        this.polling = {
                            id: '',
                            filename: '',
                            dirname: '',
                            status: false,
                            count: 0
                        };
                        clearInterval(pollingRef);
                    }
                }
            })
                .catch(e => console.log('Transformation failed!', e));
        }, 1000);
    }
    acceptFile() {
        const dataToSend = {
            uid: this.pollingCache.id,
            filename: this.pollingCache.filename,
            dirname: this.pollingCache.dirname
        };
        const reply = requestAPI('ACCEPT_FILE', {
            body: JSON.stringify(dataToSend),
            method: 'POST'
        });
        this.pollingCache = {
            id: '',
            filename: '',
            dirname: '',
            status: false,
            count: 0
        };
        reply
            .then((response) => {
            var _a;
            if (response.status === 'completed') {
                console.log('File accepted.');
                const path = (_a = this._oldFile) === null || _a === void 0 ? void 0 : _a.context.path;
                if (this._oldFile) {
                    this._oldFile.close();
                }
                if (this._newFile) {
                    this._newFile.close();
                }
                if (path) {
                    setTimeout(() => {
                        this._docManager.open(path);
                        const launcher = this._app.shell.widgets('main').next();
                        if ((launcher === null || launcher === void 0 ? void 0 : launcher.title.label) === 'Launcher') {
                            setTimeout(() => {
                                launcher.close();
                            }, 300);
                        }
                    }, 300);
                }
            }
            else {
                console.log('File accepting failed.');
            }
        })
            .catch(e => console.log('File accepting failed.', e));
    }
    declineFile() {
        const dataToSend = {
            uid: this.pollingCache.id,
            dirname: this.pollingCache.dirname
        };
        const reply = requestAPI('DECLINE_FILE', {
            body: JSON.stringify(dataToSend),
            method: 'POST'
        });
        this.pollingCache = {
            id: '',
            filename: '',
            dirname: '',
            status: false,
            count: 0
        };
        reply
            .then((response) => {
            var _a;
            if (response.status === 'completed') {
                console.log('File decline.');
                const path = (_a = this._oldFile) === null || _a === void 0 ? void 0 : _a.context.path;
                if (this._oldFile) {
                    this._oldFile.close();
                }
                if (this._newFile) {
                    this._newFile.close();
                }
                if (path) {
                    setTimeout(() => {
                        this._docManager.open(path);
                        const launcher = this._app.shell.widgets('main').next();
                        if ((launcher === null || launcher === void 0 ? void 0 : launcher.title.label) === 'Launcher') {
                            setTimeout(() => {
                                launcher.close();
                            }, 300);
                        }
                    }, 100);
                }
            }
            else {
                console.log('File declining failed.');
            }
        })
            .catch(e => console.log('File declining failed.', e));
    }
    openTransformedFile(transformedPath) {
        const options = {
            mode: 'split-right'
        };
        const oldPath = this.pollingCache.dirname + '/' + this.pollingCache.filename;
        const newPath = this.pollingCache.dirname +
            '/.' +
            this.pollingCache.id +
            '/mutableai_transform/' +
            this.pollingCache.filename;
        this._docManager.closeFile(oldPath);
        setTimeout(() => {
            this._oldFile = this._docManager.open(oldPath);
            this._newFile = this._docManager.open(newPath, undefined, undefined, options);
            if (this._newFile instanceof MainAreaWidget) {
                // Create a widget
                const widgetC = new ContentHeaderWidget({
                    onAcceptChanges: this.acceptFile,
                    onDeclineChanges: this.declineFile
                });
                // and insert it into the header
                this._newFile.contentHeader.addWidget(widgetC);
                const launcher = this._app.shell.widgets('main').next();
                if ((launcher === null || launcher === void 0 ? void 0 : launcher.title.label) === 'Launcher') {
                    setTimeout(() => {
                        launcher.close();
                    }, 300);
                }
            }
        }, 300);
    }
}
